Buffer

Struct Buffer 

pub struct Buffer<'notifier, M, L, const CAPACITY: usize>
where L: Listener<M>,
{ /* private fields */ }
Buffers messages that are to be sent through a Notifier, for efficiency.

Messages may be added to the buffer, and when the buffer contains CAPACITY messages or when it is dropped, they are all sent through Notifier::notify_many() at once. This is intended to increase performance by invoking each listener once per batch instead of once per message.

Create a Buffer by calling Notifier::buffer().

Generic parameters

  • 'notifier is the lifetime of the borrow of the Notifier which this sends messages to.
  • M is the type of message accepted by the Notifier.
  • L is the type of Listener accepted by the Notifier.
  • CAPACITY is the maximum number of messages in one batch. The buffer memory is allocated in-line in the Buffer value, so CAPACITY should be chosen with consideration for stack memory usage.

Implementations

Source

impl<'notifier, M, L, const CAPACITY: usize> Buffer<'notifier, M, L, CAPACITY>
where L: Listener<M>,

pub fn push(&mut self, message: M)

Store a message in this buffer, to be delivered later as if by Notifier::notify().

If the buffer becomes full when this message is added, then the messages in the buffer will be delivered before push() returns. Otherwise, they will be delivered when the Buffer is dropped.