pub struct Filter<F, T, const BATCH: usize> { /* private fields */ }Expand description
A Listener which transforms or discards messages before passing them on to another
Listener.
Construct this by calling Listener::filter(); see its documentation for more information.
Generic parameters
Fis the type of the filter function to use.Tis the type of the listener to pass filtered messages to.BATCHis the maximum number of filtered messages to gather before passing them on. It is used as the size of a stack-allocated array, so should be chosen with the size of the message type in mind.
Implementations
Sourceimpl<F, T> Filter<F, T, 1>
impl<F, T> Filter<F, T, 1>
Sourcepub fn with_stack_buffer<const BATCH: usize>(self) -> Filter<F, T, BATCH>
pub fn with_stack_buffer<const BATCH: usize>(self) -> Filter<F, T, BATCH>
Request that the filter accumulate output messages into a batch.
This causes each receive operation to allocate an array [MO; BATCH]
on the stack, where MO is the output message type produced by F.
Therefore, the buffer size should be chosen keeping the size of MO in mind.
Also, the amount of buffer used cannot exceed the size of the input batch,
so it is not useful to choose a buffer size larger than the expected batch size.
If MO is a zero-sized type, then the buffer is always unbounded,
so with_stack_buffer() has no effect and is unnecessary in that case.